"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElasticService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const sdk_nestjs_http_1 = require("@multiversx/sdk-nestjs-http");
const sdk_nestjs_monitoring_1 = require("@multiversx/sdk-nestjs-monitoring");
const elastic_module_options_1 = require("./entities/elastic.module.options");
let ElasticService = class ElasticService {
    constructor(options, apiService, metricsService) {
        this.options = options;
        this.apiService = apiService;
        this.metricsService = metricsService;
    }
    async getCount(collection, elasticQuery = undefined) {
        const url = `${this.options.url}/${collection}/_count`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const result = await this.post(url, elasticQuery === null || elasticQuery === void 0 ? void 0 : elasticQuery.toJson());
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.count, profiler.duration);
        const count = result.data.count;
        return count;
    }
    async getItem(collection, key, identifier) {
        var _a, _b;
        const url = `${this.options.url}/${collection}/_search?q=_id:${identifier}`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const result = await this.get(url);
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.item, profiler.duration);
        const hits = (_b = (_a = result.data) === null || _a === void 0 ? void 0 : _a.hits) === null || _b === void 0 ? void 0 : _b.hits;
        if (hits && hits.length > 0) {
            const document = hits[0];
            return this.formatItem(document, key);
        }
        return undefined;
    }
    formatItem(document, key) {
        const { _id, _source } = document;
        const item = {};
        item[key] = _id;
        return Object.assign(Object.assign({}, item), _source);
    }
    async getList(collection, key, elasticQuery, overrideUrl) {
        const url = `${overrideUrl !== null && overrideUrl !== void 0 ? overrideUrl : this.options.url}/${collection}/_search`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const result = await this.post(url, elasticQuery.toJson());
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.list, profiler.duration);
        const documents = result.data.hits.hits;
        return documents.map((document) => this.formatItem(document, key));
    }
    async getScrollableList(collection, key, elasticQuery, action, options) {
        var _a;
        const scrollTimeout = (_a = options === null || options === void 0 ? void 0 : options.scrollTimeout) !== null && _a !== void 0 ? _a : '1m';
        const url = `${this.options.url}/${collection}/_search?scroll=${scrollTimeout}`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const result = await this.post(url, elasticQuery.toJson());
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.list, profiler.duration);
        const documents = result.data.hits.hits;
        const scrollId = result.data._scroll_id;
        try {
            await action(documents.map((document) => this.formatItem(document, key)));
            while (true) {
                const scrollProfiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
                const scrollResult = await this.post(`${this.options.url}/_search/scroll`, {
                    scroll: scrollTimeout,
                    scroll_id: scrollId,
                });
                scrollProfiler.stop();
                this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.list, profiler.duration);
                const scrollDocuments = scrollResult.data.hits.hits;
                if (scrollDocuments.length === 0) {
                    break;
                }
                await action(scrollDocuments.map((document) => this.formatItem(document, key)));
                if (options === null || options === void 0 ? void 0 : options.delayBetweenScrolls) {
                    await new Promise(resolve => setTimeout(resolve, options.delayBetweenScrolls));
                }
            }
        }
        finally {
            await this.delete(`${this.options.url}/_search/scroll`, {
                scroll_id: scrollId,
            });
        }
    }
    async getCustomValue(collection, identifier, attribute) {
        var _a, _b;
        const customValuePrefix = this.options.customValuePrefix;
        if (!customValuePrefix) {
            throw new Error('Custom value prefix not defined in the elastic service options');
        }
        const url = `${this.options.url}/${collection}/_search?q=_id:${encodeURIComponent(identifier)}`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const fullAttribute = customValuePrefix + '_' + attribute;
        const payload = {
            _source: fullAttribute,
        };
        const result = await this.post(url, payload);
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.item, profiler.duration);
        const hits = (_b = (_a = result.data) === null || _a === void 0 ? void 0 : _a.hits) === null || _b === void 0 ? void 0 : _b.hits;
        if (hits && hits.length > 0) {
            const document = hits[0];
            return document._source[fullAttribute];
        }
        return null;
    }
    async setCustomValues(collection, identifier, dict) {
        const customValuePrefix = this.options.customValuePrefix;
        if (!customValuePrefix) {
            throw new Error('Custom value prefix not defined in the elastic service options');
        }
        const url = `${this.options.url}/${collection}/_update/${identifier}`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const doc = {};
        for (const [key, value] of Object.entries(dict)) {
            doc[customValuePrefix + '_' + key] = value;
        }
        const payload = { doc };
        await this.post(url, payload);
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.item, profiler.duration);
    }
    async setCustomValue(collection, identifier, attribute, value) {
        const customValuePrefix = this.options.customValuePrefix;
        if (!customValuePrefix) {
            throw new Error('Custom value prefix not defined in the elastic service options');
        }
        const url = `${this.options.url}/${collection}/_update/${identifier}`;
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        const fullAttribute = customValuePrefix + '_' + attribute;
        const payload = {
            doc: {
                [fullAttribute]: value,
            },
        };
        await this.post(url, payload);
        profiler.stop();
        this.metricsService.setElasticDuration(collection, sdk_nestjs_monitoring_1.ElasticMetricType.item, profiler.duration);
    }
    async get(url) {
        return await this.apiService.get(url);
    }
    async post(url, body) {
        return await this.apiService.post(url, body);
    }
    async delete(url, body) {
        return await this.apiService.delete(url, body);
    }
};
ElasticService = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => sdk_nestjs_http_1.ApiService))),
    tslib_1.__param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => sdk_nestjs_monitoring_1.MetricsService))),
    tslib_1.__metadata("design:paramtypes", [elastic_module_options_1.ElasticModuleOptions,
        sdk_nestjs_http_1.ApiService,
        sdk_nestjs_monitoring_1.MetricsService])
], ElasticService);
exports.ElasticService = ElasticService;
//# sourceMappingURL=elastic.service.js.map