"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElasticQuery = void 0;
const sdk_nestjs_http_1 = require("@multiversx/sdk-nestjs-http");
const query_condition_1 = require("./query.condition");
const query_condition_options_1 = require("./query.condition.options");
const query_type_1 = require("./query.type");
const range_greater_than_or_equal_1 = require("./range.greater.than.or.equal");
const range_lower_than_or_equal_1 = require("./range.lower.than.or.equal");
function buildElasticIndexerSort(sorts) {
    if (!sorts) {
        return [];
    }
    return sorts.map((sortProp) => ({ [sortProp.name]: { order: sortProp.order } }));
}
class ElasticQuery {
    constructor() {
        this.sort = [];
        this.filter = [];
        this.condition = new query_condition_1.QueryCondition();
    }
    static create() {
        return new ElasticQuery();
    }
    withFields(fields) {
        this.fields = fields;
        return this;
    }
    withPagination(pagination) {
        this.pagination = pagination;
        return this;
    }
    withSort(sort) {
        this.sort = sort;
        return this;
    }
    withMustMatchCondition(key, value, operator = undefined) {
        if (value === undefined) {
            return this;
        }
        return this.withMustCondition(query_type_1.QueryType.Match(key, value, operator));
    }
    withMustWildcardCondition(key, value) {
        if (value === undefined) {
            return this;
        }
        return this.withMustCondition(query_type_1.QueryType.Wildcard(key, `*${value.toLowerCase()}*`));
    }
    withMustMultiShouldCondition(values, action) {
        if (values === undefined) {
            return this;
        }
        return this.withMustCondition(query_type_1.QueryType.Should(values.map(value => action(value))));
    }
    withMustExistCondition(key) {
        return this.withMustCondition(query_type_1.QueryType.Exists(key));
    }
    withMustNotExistCondition(key) {
        return this.withMustNotCondition(query_type_1.QueryType.Exists(key));
    }
    withMustCondition(queries) {
        return this.withCondition(query_condition_options_1.QueryConditionOptions.must, queries);
    }
    withMustNotCondition(queries) {
        return this.withCondition(query_condition_options_1.QueryConditionOptions.mustNot, queries);
    }
    withShouldCondition(queries) {
        return this.withCondition(query_condition_options_1.QueryConditionOptions.should, queries);
    }
    withSearchWildcardCondition(search, keys) {
        return this.withSearchCondition(search, term => keys.map(key => query_type_1.QueryType.Wildcard(key, `*${term.toLowerCase()}*`)));
    }
    withSearchCondition(search, func) {
        if (!search) {
            return this;
        }
        const searchConditions = [];
        const components = search.matchAll(/\w{3,}/g);
        for (const component of components) {
            const term = component[0];
            const conditions = func(term);
            searchConditions.push(query_type_1.QueryType.Should(conditions));
        }
        return this.withMustCondition(searchConditions);
    }
    withCondition(queryCondition, queries) {
        if (!Array.isArray(queries)) {
            queries = [queries];
        }
        if (!this.condition[queryCondition]) {
            this.condition[queryCondition] = [];
        }
        this.condition[queryCondition].push(...queries);
        return this;
    }
    withTerms(termsQuery) {
        this.terms = termsQuery;
        return this;
    }
    withRangeFilter(key, ...ranges) {
        return this.withFilter(query_type_1.QueryType.Range(key, ...ranges));
    }
    withDateRangeFilter(key, before, after) {
        if (!before && !after) {
            return this;
        }
        return this.withFilter(query_type_1.QueryType.Range(key, new range_greater_than_or_equal_1.RangeGreaterThanOrEqual(after !== null && after !== void 0 ? after : 0), new range_lower_than_or_equal_1.RangeLowerThanOrEqual(before !== null && before !== void 0 ? before : Date.now())));
    }
    withFilter(filter) {
        this.filter.push(filter);
        return this;
    }
    withExtra(extra) {
        this.extra = extra;
        return this;
    }
    toJson() {
        var _a, _b;
        const elasticSort = buildElasticIndexerSort(this.sort);
        const elasticQuery = Object.assign(Object.assign(Object.assign({ _source: this.fields }, this.pagination), { sort: elasticSort, query: {
                bool: {
                    filter: this.filter.map(query => query.getQuery()),
                    must: this.condition.must.map(query => query.getQuery()),
                    should: this.condition.should.map(query => query.getQuery()),
                    must_not: this.condition.must_not.map(query => query.getQuery()),
                    minimum_should_match: this.condition.should.length !== 0 ? 1 : undefined,
                },
                terms: (_a = this.terms) === null || _a === void 0 ? void 0 : _a.getQuery(),
            } }), (_b = this.extra) !== null && _b !== void 0 ? _b : {});
        sdk_nestjs_http_1.ApiUtils.cleanupApiValueRecursively(elasticQuery);
        if (Object.keys(elasticQuery.query.bool).length === 0) {
            delete elasticQuery.query.bool;
            if (!this.terms) {
                elasticQuery.query['match_all'] = {};
            }
        }
        return elasticQuery;
    }
}
exports.ElasticQuery = ElasticQuery;
//# sourceMappingURL=elastic.query.js.map